@@ -1,5 +1,3 @@
-#!/usr/bin/python
-#
 # Copyright 2008 Dan Smith <dsmith@danplanet.com>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -15,4 +13,15 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-CHIRP_VERSION="0.1.12"
+CHIRP_VERSION="0.3.1"
+
+import os
+import sys
+from glob import glob
+
+module_dir = os.path.dirname(sys.modules["chirp"].__file__)
+__all__ = []
+for i in glob(os.path.join(module_dir, "*.py")):
+    name = os.path.basename(i)[:-3]
+    if not name.startswith("__"):
+        __all__.append(name)
