@@ -1,5 +1,3 @@
-#!/usr/bin/python
-#
 # Copyright 2008 Dan Smith <dsmith@danplanet.com>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -17,33 +15,35 @@
 
 import struct
 
-from chirp import chirp_common, util, errors
+from chirp import chirp_common, util, errors, bitwise
 from chirp.memmap import MemoryMap
 
-TUNING_STEPS = {
-    0 : 5.0,
-    1 : 6.25,
-    2 : 8.33,
-    3 : 9.0,
-    4 : 10.0,
-    5 : 12.5,
-    6 : 15,
-    7 : 20,
-    8 : 25,
-    9 : 30,
-    10: 50,
-    11: 100,
-    12: 125,
-    13: 200
-    }
+TUNING_STEPS = [
+    5.0, 6.25, 8.33,  9.0, 10.0, 12.5, 15, 20, 25, 30, 50, 100, 125, 200
+    ]
 
-TUNING_STEPS_REV = {}
-for __idx, __val in TUNING_STEPS.items():
-    TUNING_STEPS_REV[__val] = __idx
+MODES = ["FM", "NFM", "WFM", "AM", "DV"]
+DUPLEX = ["", "-", "+"]
+TMODES = ["", "Tone", "TSQL", "TSQL", "DTCS", "DTCS"]
+DTCS_POL = ["NN", "NR", "RN", "RR"]
 
 MEM_LEN = 34
 DV_MEM_LEN = 60
 
+# Dirty hack until I clean up this IC9x mess
+class IC9xMemory(chirp_common.Memory):
+    """A dirty hack to stash bank information in a memory"""
+    _bank = None
+    _bank_index = 0
+    def __init__(self):
+        chirp_common.Memory.__init__(self)
+class IC9xDVMemory(chirp_common.DVMemory):
+    """See above dirty hack"""
+    _bank = None
+    _bank_index = 0
+    def __init__(self):
+        chirp_common.DVMemory.__init__(self)
+
 def _ic9x_parse_frames(buf):
     frames = []
 
@@ -91,32 +91,34 @@ def ic9x_send(pipe, buf):
 
     return _ic9x_parse_frames(data)
 
-class IcomFrame:
-    pass
-
-class IC92Frame(IcomFrame):
+class IC92Frame:
+    """IC9x frame base class"""
     def get_vfo(self):
+        """Return the vfo number"""
         return ord(self._map[0])
 
     def set_vfo(self, vfo):
+        """Set the vfo number"""
         self._map[0] = chr(vfo)
 
     def from_raw(self, data):
+        """Construct the frame from raw data"""
         self._map = MemoryMap(data)
 
-        #self._map.printable()
-
     def from_frame(self, frame):
-        self._map = frame._map
+        """Construct the frame by copying another frame"""
+        self._map = MemoryMap(frame.get_raw())
 
     def __init__(self, subcmd=0, flen=0, cmd=0x1A):
         self._map = MemoryMap("\x00" * (4 + flen))
         self._map[0] = "\x01\x80" + chr(cmd) + chr(subcmd)
 
     def get_payload(self):
+        """Return the entire payload (sans header)"""
         return self._map[4:]
 
     def get_raw(self):
+        """Return the raw version of the frame"""
         return self._map.get_packed()
 
     def __str__(self):
@@ -128,6 +130,7 @@ class IC92Frame(IcomFrame):
         return string
 
     def send(self, pipe, verbose=False):
+        """Send the frame to the radio via @pipe"""
         if verbose:
             print "Sending:\n%s" % util.hexprint(self.get_raw())
 
@@ -148,10 +151,11 @@ class IC92Frame(IcomFrame):
         return self._map[start+4:end+4]
     
 class IC92GetBankFrame(IC92Frame):
+    """A frame for requesting bank information"""
     def __init__(self):
         IC92Frame.__init__(self, 0x09)
 
-    def send(self, pipe):
+    def send(self, pipe, verbose=False):
         rframes = ic9x_send(pipe, self.get_raw())
 
         if len(rframes) == 0:
@@ -160,27 +164,30 @@ class IC92GetBankFrame(IC92Frame):
         return rframes
 
 class IC92BankFrame(IC92Frame):
+    """A frame for bank information"""
     def __init__(self):
         # 1 byte for identifier
         # 8 bytes for name
         IC92Frame.__init__(self, 0x0B, 9)
 
-    def __str__(self):
-        return "Bank %s: %s" % (self._data[2], self._data[3:])
-
     def get_name(self):
+        """Return the bank name"""
         return self[1:]
 
     def get_identifier(self):
+        """Return the letter for the bank (A-Z)"""
         return self[0]
 
     def set_name(self, name):
+        """Set the bank name"""
         self[1] = name[:8].ljust(8)
 
     def set_identifier(self, ident):
+        """Set the letter for the bank (A-Z)"""
         self[0] = ident[0]
 
 class IC92MemClearFrame(IC92Frame):
+    """A frame for clearing (erasing) a memory"""
     def __init__(self, loc):
         # 2 bytes for location
         # 1 byte for 0xFF
@@ -189,24 +196,27 @@ class IC92MemClearFrame(IC92Frame):
         self[0] = struct.pack(">BHB", 1, int("%i" % loc, 16), 0xFF)
 
 class IC92MemGetFrame(IC92Frame):
-    def __init__(self, loc, call=False):
+    """A frame for requesting a memory"""
+    def __init__(self, loc, iscall=False):
         # 2 bytes for location
         IC92Frame.__init__(self, 0x00, 3)
 
-        if call:
-            c = 2
+        if iscall:
+            call = 2
         else:
-            c = 1
+            call = 1
 
-        self[0] = struct.pack(">BH", c, int("%i" % loc, 16))
+        self[0] = struct.pack(">BH", call, int("%i" % loc, 16))
 
 class IC92GetCallsignFrame(IC92Frame):
-    def __init__(self, type, number):
-        IC92Frame.__init__(self, type, 1, 0x1D)
+    """A frame for getting callsign information"""
+    def __init__(self, calltype, number):
+        IC92Frame.__init__(self, calltype, 1, 0x1D)
 
         self[0] = chr(number)
 
 class IC92CallsignFrame(IC92Frame):
+    """A frame to communicate callsign information"""
     command = 0 # Invalid
     width = 8
 
@@ -218,19 +228,56 @@ class IC92CallsignFrame(IC92Frame):
         self[0] = chr(number) + callsign[:self.width].ljust(self.width)
 
     def get_callsign(self):
+        """Return the actual callsign"""
         return self[1:self.width+1].rstrip()
 
 class IC92YourCallsignFrame(IC92CallsignFrame):
+    """URCALL frame"""
     command = 6 # Your
 
 class IC92RepeaterCallsignFrame(IC92CallsignFrame):
+    """RPTCALL frame"""
     command = 7 # Repeater
 
 class IC92MyCallsignFrame(IC92CallsignFrame):
+    """MYCALL frame"""
     command = 8 # My
     width = 12 # 4 bytes for /STID
 
+MEMORY_FRAME_FORMAT = """
+struct {
+  u8 vfo;
+  bbcd number[2];
+  lbcd freq[5];
+  lbcd offset[4];
+  u8 unknown8;
+  bbcd rtone[2];
+  bbcd ctone[2];
+  bbcd dtcs[2];
+  u8 unknown9[2];
+  u8 unknown2:1,
+     mode:3,
+     tuning_step:4;
+  u8 unknown1:3,
+     tmode: 3,
+     duplex: 2;
+  u8 unknown5:4,
+     dtcs_polarity:2,
+     pskip:1,
+     skip:1;
+  char bank;
+  bbcd bank_index[1];
+  char name[8];
+  u8 unknown10;
+  u8 digital_code;
+  char rpt2call[8];
+  char rpt1call[8];
+  char urcall[8];
+} mem[1];
+"""
+
 class IC92MemoryFrame(IC92Frame):
+    """A frame for communicating memory information"""
     def __init__(self):
         IC92Frame.__init__(self, 0, DV_MEM_LEN)
 
@@ -254,313 +301,147 @@ class IC92MemoryFrame(IC92Frame):
             self._map.truncate(MEM_LEN + 4)
 
     def set_iscall(self, iscall):
+        """This frame refers to a call channel if @iscall is True"""
         if iscall:
             self[0] = 2
         else:
             self[0] = 1
 
     def get_iscall(self):
+        """Return True if this frame refers to a call channel"""
         return ord(self[0]) == 2
 
-    def _encode_duptone(self, mem):
-        duptone = ord(self[22]) & 0xE0
-
-        if mem.duplex == "-":
-            duptone |= duptone | 0x01
-        elif mem.duplex == "+":
-            duptone |= duptone | 0x02
-
-        if mem.tmode == "Tone":
-            duptone |= 0x04
-        elif mem.tmode == "TSQL":
-            duptone |= 0x0C
-        elif mem.tmode == "DTCS":
-            duptone |= 0x14
-
-        self[22] = duptone
-
-    def _decode_duptone(self):
-        duptone = ord(self[22])
-
-        if duptone & 0x01:
-            duplex = "-"
-        elif duptone & 0x02:
-            duplex = "+"
-        else:
-            duplex = ""
-
-        if (duptone & 0x0C) == 0x0C:
-            tmode = "TSQL"
-        elif (duptone & 0x14) == 0x14:
-            tmode = "DTCS"
-        elif (duptone & 0x04) == 0x04:
-            tmode = "Tone"
-        else:
-            tmode = ""
-
-        return duplex, tmode
-
-    def _encode_mode(self, mem):
-        mode = ord(self[21]) & 0x0F
-
-        if mem.mode == "FM":
-            pass
-        elif mem.mode == "NFM":
-            mode |= 0x10
-        elif mem.mode == "WFM":
-            mode |= 0x20
-        elif mem.mode == "AM":
-            mode |= 0x30
-        elif mem.mode == "DV":
-            mode |= 0x40
-        else:
-            raise errors.InvalidDataError("Unsupported mode %s" % mem.mode)
-
-        self[21] = mode
-
-    def _decode_mode(self):
-        mode = ord(self[21])
-
-        if (mode & 0x30) == 0x30:
-            return "AM"
-        elif (mode & 0x10) == 0x10:
-            return "NFM"
-        elif (mode & 0x20) == 0x20:
-            return "WFM"
-        elif (mode & 0x40) == 0x40:
-            return "DV"
-        else:
-            return "FM"
-
-    def _encode_dtcs_polarity(self, mem):
-        pol = ord(self[23]) & 0xF3
-        if mem.dtcs_polarity[0] == "R":
-            pol |= 0x80
-        if mem.dtcs_polarity[1] == "R":
-            pol |= 0x40
-        self[23] = pol
-
-    def _decode_dtcs_polarity(self):
-        pol = ord(self[23])
-
-        pstr = ""
-        if pol & 0x80:
-            pstr += "R"
-        else:
-            pstr += "N"
-        if pol & 0x40:
-            pstr += "R"
-        else:
-            pstr += "N"
-
-        return pstr
-
-    def _encode_tuning_step(self, mem):
-        ts = ord(self[21]) & 0xF0
-        ts |= TUNING_STEPS_REV[mem.tuning_step]
-        self[21] = ts
-
-    def _decode_tuning_step(self):
-        ts = ord(self[21]) & 0x0F
-        return TUNING_STEPS[ts]
-
-    def _encode_skip(self, mem):
-        skip = ord(self[23]) & 0xFC
-        if mem.skip == "S":
-            skip |= 0x01
-        elif mem.skip == "P":
-            skip |= 0x02
-        self[23] = skip
-
-    def _decode_skip(self):
-        skip = ord(self[23])
-
-        if skip & 0x01:
-            return "S"
-        elif skip & 0x02:
-            return "P"
-        else:
-            return ""
-
-    def _encode_bank(self, mem):
-        if mem.bank is None:
-            self[24] = 0
-        else:
-            self[24] = chr(mem.bank + ord("A"))
-
-        if mem.bank_index == -1:
-            self[25] = util.bcd_encode(0)
-        else:
-            self[25] = util.bcd_encode(mem.bank_index)
-
-    def _decode_bank(self):
-        if ord(self[24]) == 0 or self.get_iscall():
-            bank = None
-            index = -1
-        else:
-            bank = ord(self[24]) - ord("A")
-            index = int("%02x" % ord(self[25]))
-
-        return bank, index
-
-    def _encode_calls(self, mem):
-        if isinstance(mem, chirp_common.DVMemory):
-            uc = mem.dv_urcall
-            r1 = mem.dv_rpt1call
-            r2 = mem.dv_rpt2call
-        else:
-            uc = r1 = r2 = (" " * 8)
-
-        self[36] = r2.ljust(8)
-        self[44] = r1.ljust(8)
-        self[52] = uc.ljust(8)
-
-    def _decode_calls(self):
-        return self[52:60].rstrip(), self[44:52].rstrip(),self[36:44].rstrip()
-
-    def _decode_digital_code(self):
-        return int("%02x" % ord(self[35]))
-
-    def _encode_digital_code(self, mem):
-        self[35] = util.bcd_encode(mem.dv_code)
-
-    def _decode_freq(self):
-        return int("%02x%02x%02x%02x%02x" % (ord(self[7]),
-                                             ord(self[6]),
-                                             ord(self[5]),
-                                             ord(self[4]),
-                                             ord(self[3])))
-
     def set_memory(self, mem):
+        """Take Memory object @mem and configure the frame accordingly"""
         if mem.number < 0:
             self.set_iscall(True)
             mem.number = abs(mem.number) - 1
             print "Memory is %i (call %s)" % (mem.number, self.get_iscall())
 
-        self[1] = struct.pack(">H", int("%i" % mem.number, 16))
+        _mem = bitwise.parse(MEMORY_FRAME_FORMAT, self).mem
+
+        _mem.number = mem.number
+
+        _mem.freq = mem.freq
+        _mem.offset = mem.offset
+        _mem.rtone = int(mem.rtone * 10)
+        _mem.ctone = int(mem.ctone * 10)
+        _mem.dtcs = int(mem.dtcs)
+        _mem.mode = MODES.index(mem.mode)
+        _mem.tuning_step = TUNING_STEPS.index(mem.tuning_step)
+        _mem.duplex = DUPLEX.index(mem.duplex)
+        _mem.tmode = TMODES.index(mem.tmode)
+        _mem.dtcs_polarity = DTCS_POL.index(mem.dtcs_polarity)
+
+        if mem._bank is not None:
+            _mem.bank = chr(ord("A") + mem._bank)
+            _mem.bank_index = mem._bank_index
+
+        _mem.skip = mem.skip == "S"
+        _mem.pskip = mem.skip == "P"
+
+        _mem.name = mem.name.ljust(8)[:8]
 
-        self[3] = util.bcd_encode(mem.freq,
-                                  bigendian=False,
-                                  width=10)
-        self[8] = util.bcd_encode(mem.offset,
-                                  bigendian=False,
-                                  width=6)
-        self[13] = util.bcd_encode(int(mem.rtone * 10))
-        self[15] = util.bcd_encode(int(mem.ctone * 10))
-        self[17] = util.bcd_encode(int(mem.dtcs), width=4)
-        self._encode_mode(mem)
-        self._encode_tuning_step(mem)
-        self._encode_duptone(mem)
-        self._encode_dtcs_polarity(mem)
-        self._encode_skip(mem)
-        self._encode_bank(mem)
         if mem.mode == "DV":
-            self._encode_calls(mem)
-            self._encode_digital_code(mem)
-
-        self[26] = mem.name[:8].ljust(8)
+            _mem.urcall = mem.dv_urcall.upper().ljust(8)[:8]
+            _mem.rpt1call = mem.dv_rpt1call.upper().ljust(8)[:8]
+            _mem.rpt2call = mem.dv_rpt2call.upper().ljust(8)[:8]
+            _mem.digital_code = mem.dv_code
 
     def get_memory(self):
-        if self._decode_mode() == "DV":
-            mem = chirp_common.DVMemory()
+        """Return a Memory object based on the contents of the frame"""
+        _mem = bitwise.parse(MEMORY_FRAME_FORMAT, self).mem
+
+        if MODES[_mem.mode] == "DV":
+            mem = IC9xDVMemory()
         else:
-            mem = chirp_common.Memory()
-
-        mem.number = int("%02x" % struct.unpack(">H", self[1:3])[0])
+            mem = IC9xMemory()
 
+        mem.number = int(_mem.number)
         if self.get_iscall():
             mem.number = -1 - mem.number
 
-        mem.freq = self._decode_freq()
-        mem.offset = float("%02x%02x%02x%02x" % (ord(self[11]), ord(self[10]),
-                                                 ord(self[9]),  ord(self[8])))
-        mem.rtone = int("%02x%02x" % (ord(self[13]), ord(self[14]))) / 10.0
-        mem.ctone = int("%02x%02x" % (ord(self[15]), ord(self[16]))) / 10.0
-        mem.dtcs = int("%02x%02x"  % (ord(self[17]), ord(self[18])))
-        mem.mode = self._decode_mode()
-        mem.tuning_step = self._decode_tuning_step()
-        mem.duplex, mem.tmode = self._decode_duptone()
-        mem.dtcs_polarity = self._decode_dtcs_polarity()
-        mem.skip = self._decode_skip()
-        mem.bank, mem.bank_index = self._decode_bank()
+        mem.freq = int(_mem.freq)
+        mem.offset = int(_mem.offset)
+        mem.rtone = int(_mem.rtone) / 10.0
+        mem.ctone = int(_mem.ctone) / 10.0
+        mem.dtcs = int(_mem.dtcs)
+        mem.mode = MODES[int(_mem.mode)]
+        mem.tuning_step = TUNING_STEPS[int(_mem.tuning_step)]
+        mem.duplex = DUPLEX[int(_mem.duplex)]
+        mem.tmode = TMODES[int(_mem.tmode)]
+        mem.dtcs_polarity = DTCS_POL[int(_mem.dtcs_polarity)]
+
+        if int(_mem.bank) != 0:
+            mem._bank = ord(str(_mem.bank)) - ord("A")
+            mem._bank_index = int(_mem.bank_index)
+
+        if _mem.skip:
+            mem.skip = "S"
+        elif _mem.pskip:
+            mem.skip = "P"
+        else:
+            mem.skip = ""
+
+        mem.name = str(_mem.name).rstrip()
 
         if mem.mode == "DV":
-            mem.dv_urcall, mem.dv_rpt1call, mem.dv_rpt2call = \
-                self._decode_calls()
-            mem.dv_code = self._decode_digital_code()
-
-        mem.name = self[26:34].rstrip()
+            mem.dv_urcall = str(_mem.urcall).rstrip()
+            mem.dv_rpt1call = str(_mem.rpt1call).rstrip()
+            mem.dv_rpt2call = str(_mem.rpt2call).rstrip()
+            mem.dv_code = int(_mem.digital_code)
 
         return mem
 
-def print_frames(frames):
-    count = 0
-    for i in frames:
-        print "Frame %i:" % count
-        print i
-        count += 1
-
 def _send_magic_4800(pipe):
     cmd = "\x01\x80\x19"
     magic = ("\xFE" * 25) + cmd
-    for i in [0,1]:
-        r = ic9x_send(pipe, magic)
-        if r:
-            return r[0].get_raw()[0] == "\x80"
-    return r and r[0].get_raw()[:3] == rsp
+    for _i in [0, 1]:
+        resp = ic9x_send(pipe, magic)
+        if resp:
+            return resp[0].get_raw()[0] == "\x80"
+    return True
 
 def _send_magic_38400(pipe):
     cmd = "\x01\x80\x19"
-    rsp = "\x80\x01\x19"
+    #rsp = "\x80\x01\x19"
     magic = ("\xFE" * 400) + cmd
-    for i in [0,1]:
-        r = ic9x_send(pipe, magic)
-        if r:
-            return r[0].get_raw()[0] == "\x80"
+    for _i in [0, 1]:
+        resp = ic9x_send(pipe, magic)
+        if resp:
+            return resp[0].get_raw()[0] == "\x80"
     return False
 
 def send_magic(pipe):
+    """Send the magic incantation to wake up an ic9x radio"""
     if pipe.getBaudrate() == 38400:
-        r = _send_magic_38400(pipe)
-        if r:
+        resp = _send_magic_38400(pipe)
+        if resp:
             return
         print "Switching from 38400 to 4800"
         pipe.setBaudrate(4800)
-        r = _send_magic_4800(pipe)
+        resp = _send_magic_4800(pipe)
         pipe.setBaudrate(38400)
-        if r:
+        if resp:
             return
         raise errors.RadioError("Radio not responding")
     elif pipe.getBaudrate() == 4800:
-        r = _send_magic_4800(pipe)
-        if r:
+        resp = _send_magic_4800(pipe)
+        if resp:
             return
         print "Switching from 4800 to 38400"
         pipe.setBaudrate(38400)
-        r = _send_magic_38400(pipe)
-        if r:
+        resp = _send_magic_38400(pipe)
+        if resp:
             return
         pipe.setBaudrate(4800)
         raise errors.RadioError("Radio not responding")
     else:
-        raise errors.InvalidDataError("Radio in unknown state (%i)" % r.getBaudrate())    
-
-def print_banks(pipe):
-    frames = send(pipe, "\x01\x80\x1a\x09") # Banks
-
-    print "A Banks:"
-    for i in range(180, 180+26):
-        bf = IC92BankFrame()
-        bf.from_frame(frames[i])
-        print str(bf)
-
-    print "B Banks:"
-    for i in range(237, 237+26):
-        bf = IC92BankFrame()
-        bf.from_frame(frames[i])
-        print str(bf)
+        raise errors.InvalidDataError("Radio in unknown state (%i)" % \
+                                          pipe.getBaudrate())    
 
 def get_memory_frame(pipe, vfo, number):
+    """Get the memory frame for @vfo and @number via @pipe"""
     if number < 0:
         number = abs(number + 1)
         call = True
@@ -573,6 +454,7 @@ def get_memory_frame(pipe, vfo, number):
     return frame.send(pipe)
 
 def get_memory(pipe, vfo, number):
+    """Get a memory object for @vfo and @number via @pipe"""
     rframe = get_memory_frame(pipe, vfo, number)
 
     if len(rframe.get_payload()) < 1:
@@ -587,6 +469,7 @@ def get_memory(pipe, vfo, number):
     return mf.get_memory()
 
 def set_memory(pipe, vfo, memory):
+    """Set memory @memory on @vfo via @pipe"""
     frame = IC92MemoryFrame()
     frame.set_memory(memory)
     frame.set_vfo(vfo)
@@ -601,6 +484,7 @@ def set_memory(pipe, vfo, memory):
                                           util.hexprint(rframe.get_payload()))
 
 def erase_memory(pipe, vfo, number):
+    """Erase memory @number on @vfo via @pipe"""
     frame = IC92MemClearFrame(number)
     frame.set_vfo(vfo)
 
@@ -609,6 +493,7 @@ def erase_memory(pipe, vfo, number):
         raise errors.InvalidDataError("Radio reported error")
 
 def get_banks(pipe, vfo):
+    """Get banks for @vfo via @pipe"""
     frame = IC92GetBankFrame()
     frame.set_vfo(vfo)
 
@@ -625,12 +510,12 @@ def get_banks(pipe, vfo):
         bframe = IC92BankFrame()
         bframe.from_frame(rframes[i])
 
-        bank = chirp_common.Bank(bframe.get_name())
-        banks.append(bank)
+        banks.append(bframe.get_name().rstrip())
     
     return banks
 
 def set_banks(pipe, vfo, banks):
+    """Set banks for @vfo via @pipe"""
     for i in range(0, 26):
         bframe = IC92BankFrame()
         bframe.set_vfo(vfo)
@@ -642,6 +527,7 @@ def set_banks(pipe, vfo, banks):
             raise errors.InvalidDataError("Radio reported error")
 
 def get_call(pipe, cstype, number):
+    """Get @cstype callsign @number via @pipe"""
     cframe = IC92GetCallsignFrame(cstype.command, number)
     cframe.set_vfo(2)
     rframe = cframe.send(pipe)
@@ -652,28 +538,10 @@ def get_call(pipe, cstype, number):
     return cframe.get_callsign()
 
 def set_call(pipe, cstype, number, call):
+    """Set @cstype @call at position @number via @pipe"""
     cframe = cstype(number, call)
     cframe.set_vfo(2)
     rframe = cframe.send(pipe)
 
     if rframe.get_payload() != "\xfb":
         raise errors.RadioError("Radio reported error")
-
-def print_memory(pipe, vfo, number):
-    if vfo not in [1, 2]:
-        raise errors.InvalidValueError("VFO must be 1 or 2")
-
-    if number < 0 or number > 399:
-        raise errors.InvalidValueError("Number must be between 0 and 399")
-
-    mf = get_memory(pipe, vfo, number)
-
-    print "Memory %i from VFO %i: %s" % (number, vfo, str(mf))
-
-if __name__ == "__main__":
-    print util.hexprint(util.bcd_encode(1072))
-    print util.hexprint(util.bcd_encode(146900000, False))
-    print util.hexprint(util.bcd_encode(25, width=4))
-    print util.hexprint(util.bcd_encode(5000000, False, 6))
-    print util.hexprint(util.bcd_encode(600000, False, 6))
-    
