@@ -1,5 +1,3 @@
-#!/usr/bin/python
-#
 # Copyright 2008 Dan Smith <dsmith@danplanet.com>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -19,7 +17,7 @@ import gtk
 import gobject
 import pango
 
-from chirp import errors, chirp_common, xml, import_logic
+from chirp import errors, chirp_common, generic_xml, import_logic
 from chirpui import common
 
 class WaitWindow(gtk.Window):
@@ -58,6 +56,19 @@ class WaitWindow(gtk.Window):
 
         self.prog.set_fraction(fraction)
 
+class ImportMemoryBankJob(common.RadioJob):
+    def __init__(self, cb, dst_mem, src_radio, src_mem):
+        common.RadioJob.__init__(self, cb, None)
+        self.__dst_mem = dst_mem
+        self.__src_radio = src_radio
+        self.__src_mem = src_mem
+
+    def execute(self, radio):
+        import_logic.import_bank(radio, self.__src_radio,
+                                 self.__dst_mem, self.__src_mem)
+        if self.cb:
+            gobject.idle_add(self.cb, *self.cb_args)
+
 class ImportDialog(gtk.Dialog):
 
     def _check_for_dupe(self, location):
@@ -76,9 +87,10 @@ class ImportDialog(gtk.Dialog):
         if not imp and self._check_for_dupe(nloc):
             d = gtk.MessageDialog(parent=self, buttons=gtk.BUTTONS_OK)
             d.set_property("text",
-                           "Location %i is already being imported.  " % nloc + \
-                               "Choose another value for 'New Location' " + \
-                               "before selecting 'Import'")
+                           _("Location {number} is already being imported. "
+                             "Choose another value for 'New Location' "
+                             "before selection 'Import'").format(\
+                    number=nloc))
             d.run()
             d.destroy()
         else:
@@ -101,39 +113,50 @@ class ImportDialog(gtk.Dialog):
 
     def _edited(self, rend, path, new, col):
         iter = self.__store.get_iter(path)
-        nloc, = self.__store.get(iter, self.col_nloc)
+        
+        if col == self.col_nloc:
+            nloc, = self.__store.get(iter, self.col_nloc)
 
-        try:
-            val = int(new)
-        except ValueError:
-            common.show_error("Invalid value.  Must be an integer.")
+            try:
+                val = int(new)
+            except ValueError:
+                common.show_error(_("Invalid value. Must be an integer."))
+                return
+
+            if val == nloc:
+                return
+
+            if self._check_for_dupe(val):
+                d = gtk.MessageDialog(parent=self, buttons=gtk.BUTTONS_OK)
+                d.set_property("text",
+                               _("Location {number} is already being "
+                                 "imported").format(number=val))
+                d.run()
+                d.destroy()
+                return
+
+            self.record_use_of(val)
+
+        elif col == self.col_name or col == self.col_comm:
+            val = str(new)
+
+        else:
             return
 
-        if val == nloc:
-            return
-
-        if self._check_for_dupe(val):
-            d = gtk.MessageDialog(parent=self, buttons=gtk.BUTTONS_OK)
-            d.set_property("text",
-                           "Location %i is already being imported" % val)
-            d.run()
-            d.destroy()
-            return
-
-        self.record_use_of(val)
-
         self.__store.set(iter, col, val)
 
     def get_import_list(self):
         import_list = []
         iter = self.__store.get_iter_first()
         while iter:
-            old, new, enb = self.__store.get(iter,
+            old, new, name, comm, enb = self.__store.get(iter,
                                              self.col_oloc,
                                              self.col_nloc,
+                                             self.col_name,
+                                             self.col_comm,
                                              self.col_import)
             if enb:
-                import_list.append((old, new))
+                import_list.append((old, new, name, comm))
             iter = self.__store.iter_next(iter)
 
         return import_list
@@ -166,12 +189,12 @@ class ImportDialog(gtk.Dialog):
                 
         if ulist_changed:
             job = common.RadioJob(None, "set_urcall_list", ulist)
-            job.set_desc("Updating URCALL list")
+            job.set_desc(_("Updating URCALL list"))
             dst_rthread._qsubmit(job, 0)
 
         if rlist_changed:
             job = common.RadioJob(None, "set_repeater_call_list", ulist)
-            job.set_desc("Updating RPTCALL list")
+            job.set_desc(_("Updating RPTCALL list"))
             dst_rthread._qsubmit(job, 0)
             
         return
@@ -212,7 +235,7 @@ class ImportDialog(gtk.Dialog):
             print "One or more of the radios doesn't support banks"
             return
 
-        if not isinstance(self.dst_radio, xml.XMLRadio) and \
+        if not isinstance(self.dst_radio, generic_xml.XMLRadio) and \
                 len(dst_banks) != len(src_banks):
             print "Source and destination radios have a different number of banks"
         else:
@@ -223,7 +246,9 @@ class ImportDialog(gtk.Dialog):
         error_messages = {}
         import_list = self.get_import_list()
 
-        for old, new in import_list:
+        src_features = self.src_radio.get_features()
+
+        for old, new, name, comm in import_list:
             i += 1
             print "%sing %i -> %i" % (self.ACTION, old, new)
 
@@ -231,20 +256,26 @@ class ImportDialog(gtk.Dialog):
 
             try:
                 mem = import_logic.import_mem(self.dst_radio,
-                                              self.src_radio,
+                                              src_features,
                                               src,
-                                             {"number" : new})
+                                              {"number" : new,
+                                               "name"   : name,
+                                               "comment": comm})
             except import_logic.ImportError, e:
                 print e
                 error_messages[new] = str(e)
                 continue
 
             job = common.RadioJob(None, "set_memory", mem)
-            job.set_desc("Setting memory %i" % mem.number)
+            job.set_desc(_("Setting memory {number}").format(number=mem.number))
+            dst_rthread._qsubmit(job, 0)
+
+            job = ImportMemoryBankJob(None, mem, self.src_radio, src)
+            job.set_desc(_("Importing bank information"))
             dst_rthread._qsubmit(job, 0)
 
         if error_messages.keys():
-            msg = "Error importing memories:\r\n"
+            msg = _("Error importing memories:") + "\r\n"
             for num, msgs in error_messages.items():
                 msg += "%s: %s" % (num, ",".join(msgs))
             common.show_error(msg)
@@ -252,7 +283,7 @@ class ImportDialog(gtk.Dialog):
         return i
 
     def make_view(self):
-        editable = [self.col_nloc]
+        editable = [self.col_nloc, self.col_name, self.col_comm]
 
         self.__store = gtk.ListStore(gobject.TYPE_BOOLEAN, # Import
                                      gobject.TYPE_INT,     # Source loc
@@ -265,6 +296,8 @@ class ImportDialog(gtk.Dialog):
         self.__view = gtk.TreeView(self.__store)
         self.__view.show()
 
+        tips = gtk.Tooltips()
+
         for k in self.caps.keys():
             t = self.types[k]
 
@@ -287,8 +320,16 @@ class ImportDialog(gtk.Dialog):
             if k == self.col_nloc:
                 column.set_cell_data_func(rend, self._render, k)
 
+            if k in self.tips.keys():
+                print "Doing %s" % k
+                lab = gtk.Label(self.caps[k])
+                column.set_widget(lab)
+                tips.set_tip(lab, self.tips[k])
+                lab.show()
             column.set_sort_column_id(k)
             self.__view.append_column(column)
+
+        self.__view.set_tooltip_column(self.col_tmsg)
         
         sw = gtk.ScrolledWindow()
         sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
@@ -346,25 +387,25 @@ class ImportDialog(gtk.Dialog):
     def make_select(self):
         hbox = gtk.HBox(True, 2)
 
-        all = gtk.Button("All");
+        all = gtk.Button(_("All"));
         all.connect("clicked", self.__select_all, True)
         all.set_size_request(50, 25)
         all.show()
         hbox.pack_start(all, 0, 0, 0)
 
-        none = gtk.Button("None");
+        none = gtk.Button(_("None"));
         none.connect("clicked", self.__select_all, False)
         none.set_size_request(50, 25)
         none.show()
         hbox.pack_start(none, 0, 0, 0)
 
-        inv = gtk.Button("Inverse")
+        inv = gtk.Button(_("Inverse"))
         inv.connect("clicked", self.__select_all, None)
         inv.set_size_request(50, 25)
         inv.show()
         hbox.pack_start(inv, 0, 0, 0)
 
-        frame = gtk.Frame("Select")
+        frame = gtk.Frame(_("Select"))
         frame.show()
         frame.add(hbox)
         hbox.show()
@@ -374,6 +415,18 @@ class ImportDialog(gtk.Dialog):
     def make_adjust(self):
         hbox = gtk.HBox(True, 2)
 
+        incr = gtk.Button("+100")
+        incr.connect("clicked", self.__incrnew, 100)
+        incr.set_size_request(50, 25)
+        incr.show()
+        hbox.pack_start(incr, 0, 0, 0)
+
+        incr = gtk.Button("+10")
+        incr.connect("clicked", self.__incrnew, 10)
+        incr.set_size_request(50, 25)
+        incr.show()
+        hbox.pack_start(incr, 0, 0, 0)
+
         incr = gtk.Button("+1")
         incr.connect("clicked", self.__incrnew, 1)
         incr.set_size_request(50, 25)
@@ -386,19 +439,31 @@ class ImportDialog(gtk.Dialog):
         decr.show()
         hbox.pack_start(decr, 0, 0, 0)
 
-        auto = gtk.Button("Auto")
+        decr = gtk.Button("-10")
+        decr.connect("clicked", self.__incrnew, -10)
+        decr.set_size_request(50, 25)
+        decr.show()
+        hbox.pack_start(decr, 0, 0, 0)
+
+        decr = gtk.Button("-100")
+        decr.connect("clicked", self.__incrnew, -100)
+        decr.set_size_request(50, 25)
+        decr.show()
+        hbox.pack_start(decr, 0, 0, 0)
+
+        auto = gtk.Button(_("Auto"))
         auto.connect("clicked", self.__autonew)
         auto.set_size_request(50, 25)
         auto.show()
         hbox.pack_start(auto, 0, 0, 0)
 
-        revr = gtk.Button("Reverse")
+        revr = gtk.Button(_("Reverse"))
         revr.connect("clicked", self.__revrnew)
         revr.set_size_request(50, 25)
         revr.show()
         hbox.pack_start(revr, 0, 0, 0)
 
-        frame = gtk.Frame("Adjust New Location")
+        frame = gtk.Frame(_("Adjust New Location"))
         frame.show()
         frame.add(hbox)
         hbox.show()
@@ -408,13 +473,13 @@ class ImportDialog(gtk.Dialog):
     def make_options(self):
         hbox = gtk.HBox(True, 2)
 
-        confirm = gtk.CheckButton("Confirm overwrites")
+        confirm = gtk.CheckButton(_("Confirm overwrites"))
         confirm.connect("toggled", __set_confirm)
         confirm.show()
 
         hbox.pack_start(confirm, 0, 0, 0)
 
-        frame = gtk.Frame("Options")
+        frame = gtk.Frame(_("Options"))
         frame.add(hbox)
         frame.show()
         hbox.show()
@@ -422,7 +487,7 @@ class ImportDialog(gtk.Dialog):
         return frame
 
     def make_controls(self):
-        hbox = gtk.HBox(True, 2)
+        hbox = gtk.HBox(False, 2)
         
         hbox.pack_start(self.make_select(), 0, 0, 0)
         hbox.pack_start(self.make_adjust(), 0, 0, 0)
@@ -454,21 +519,42 @@ class ImportDialog(gtk.Dialog):
     def populate_list(self):
         start, end = self.src_radio.get_features().memory_bounds
         for i in range(start, end+1):
-            self.ww.set(float(i) / end)
+            if end > 50 and i % (end/50) == 0:
+                self.ww.set(float(i) / end)
             try:
                 mem = self.src_radio.get_memory(i)
             except errors.InvalidMemoryLocation, e:
                 continue
+            except Exception, e:
+                self.__store.append(row=(False,
+                                         i,
+                                         i,
+                                         "ERROR",
+                                         chirp_common.format_freq(0),
+                                         "",
+                                         False,
+                                         str(e),
+                                         ))
+                self.record_use_of(i)
+                continue
             if mem.empty:
                 continue
 
-            msgs = self.dst_radio.validate_memory(mem)
+            self.ww.set(float(i) / end)
+            try:
+                msgs = self.dst_radio.validate_memory(
+                        import_logic.import_mem(self.dst_radio,
+                                                self.src_radio.get_features(),
+                                                mem))
+            except import_logic.DestNotCompatible:
+                msgs = self.dst_radio.validate_memory(mem)
             errs = [x for x in msgs if isinstance(x, chirp_common.ValidationError)]
             if errs:
-                msg = "Unsupported by destination radio: %s" % (",".join(msgs))
+                msg = _("Cannot be imported because") + ":\r\n"
+                msg += ",".join(errs)
             else:
                 errs = []
-                msg = str(mem)
+                msg = "Memory can be imported into target"
 
             self.__store.append(row=(not bool(msgs),
                                      mem.number,
@@ -482,8 +568,8 @@ class ImportDialog(gtk.Dialog):
             self.record_use_of(mem.number)
 
 
-    TITLE = "Import From File"
-    ACTION = "Import"
+    TITLE = _("Import From File")
+    ACTION = _("Import")
 
     def __init__(self, src_radio, dst_radio, parent=None):
         gtk.Dialog.__init__(self,
@@ -503,11 +589,16 @@ class ImportDialog(gtk.Dialog):
 
         self.caps = {
             self.col_import : self.ACTION,
-            self.col_nloc   : "New location",
-            self.col_oloc   : "Location",
-            self.col_name   : "Name",
-            self.col_freq   : "Frequency",
-            self.col_comm   : "",
+            self.col_nloc   : _("To"),
+            self.col_oloc   : _("From"),
+            self.col_name   : _("Name"),
+            self.col_freq   : _("Frequency"),
+            self.col_comm   : _("Comment"),
+            }
+
+        self.tips = {
+            self.col_nloc : _("Location memory will be imported into"),
+            self.col_oloc : _("Location of memory in the file being imported"),
             }
 
         self.types = {
@@ -528,9 +619,9 @@ class ImportDialog(gtk.Dialog):
         self.not_used_list = []
 
         self.build_ui()
-        self.set_default_size(400, 300)
+        self.set_default_size(600, 400)
 
-        self.ww = WaitWindow("Preparing memory list...", parent=parent)
+        self.ww = WaitWindow(_("Preparing memory list..."), parent=parent)
         self.ww.show()
         self.ww.grind()
 
@@ -539,8 +630,8 @@ class ImportDialog(gtk.Dialog):
         self.ww.hide()
 
 class ExportDialog(ImportDialog):
-    TITLE = "Export To File"
-    ACTION = "Export"
+    TITLE = _("Export To File")
+    ACTION = _("Export")
 
 if __name__ == "__main__":
     from chirpui import editorset
